"""
Pytest Fixtures Shared Across all Unit Tests
"""

import pathlib
from typing import Any, Dict, List

import pytest
from upath import UPath

from tests.helpers import Screenshotter


@pytest.fixture
def repo_dir() -> pathlib.Path:
    """
    Return the path to the repository root
    """
    return pathlib.Path(__file__).parent.parent.resolve()


@pytest.fixture
def screenshot_dir(repo_dir: pathlib.Path) -> pathlib.Path:
    """
    Return the path to the screenshot directory
    """
    return repo_dir / "tests" / "screenshots"


@pytest.fixture
def github_release_path() -> UPath:
    """
    Return the path to the GitHub Release
    """
    release = "v1.0.0"
    uri = f"github://juftin:textual-universal-directorytree@{release}"
    return UPath(uri)


@pytest.fixture(scope="module")
def vcr_config() -> Dict[str, List[Any]]:
    """
    VCR Cassette Privacy Enforcer

    This fixture ensures the API Credentials are obfuscated

    Returns
    -------
    Dict[str, List[Any]]:
        The VCR Config
    """
    return {
        "filter_headers": [("authorization", "XXXXXXXXXX")],
        "filter_query_parameters": [("user", "XXXXXXXXXX"), ("token", "XXXXXXXXXX")],
    }


@pytest.fixture
def screenshotter(github_release_path: UPath) -> Screenshotter:
    """
    Return a Screenshotter

    Parameters
    ----------
    github_release_path : UPath
        The path to the GitHub Release

    Returns
    -------
    Screenshotter:
        The Screenshotter
    """
    return Screenshotter(file_path=github_release_path)


cassette = pytest.mark.vcr(scope="module")



"""
Textual Universal Directory Tree
"""

from upath import UPath, registry

from textual_universal_directorytree.alternate_paths import (
    GitHubTextualPath,
    S3TextualPath,
)
from textual_universal_directorytree.universal_directory_tree import (
    UniversalDirectoryTree,
)
from textual_universal_directorytree.utils import is_local_path, is_remote_path

registry.register_implementation(protocol="github", cls=GitHubTextualPath, clobber=True)
registry.register_implementation(protocol="s3", cls=S3TextualPath, clobber=True)
registry.register_implementation(protocol="s3a", cls=S3TextualPath, clobber=True)

__all__ = [
    "UniversalDirectoryTree",
    "is_local_path",
    "is_remote_path",
    "GitHubTextualPath",
    "S3TextualPath",
    "UPath",
]



"""
Textual's Implementation of Pathlib, Powered by fsspec
"""

from __future__ import annotations

import os
from typing import Any

from upath.implementations.cloud import S3Path
from upath.implementations.github import GitHubPath


class GitHubTextualPath(GitHubPath):
    """
    GitHubPath

    UPath implementation for GitHub to be compatible with the Directory Tree
    """

    @classmethod
    def _transform_init_args(
        cls,
        args: tuple[str | os.PathLike[Any], ...],
        protocol: str,
        storage_options: dict[str, Any],
    ) -> tuple[tuple[str | os.PathLike[Any], ...], str, dict[str, Any]]:
        """
        Initialize the GitHubPath with GitHub Token Authentication
        """
        if "token" not in storage_options:
            token = os.getenv("GITHUB_TOKEN")
            if token is not None:
                storage_options.update({"username": "Bearer", "token": token})
        handled_args = args
        if "sha" not in storage_options:
            handled_url = cls.handle_github_url(args[0])
            handled_args = (handled_url, *args[1:])
        return handled_args, protocol, storage_options

    def __str__(self) -> str:
        """
        String representation of the GitHubPath
        """
        return (
            f"{self.protocol}://{self.storage_options['org']}:"
            f"{self.storage_options['repo']}@{self.storage_options['sha']}/"
            f"{self.path}"
        )

    @classmethod
    def handle_github_url(cls, url: str | GitHubPath) -> str:
        """
        Handle GitHub URLs

        GitHub URLs are handled by converting them to the raw URL.
        """
        try:
            import requests
        except ImportError as e:
            raise ImportError(
                "The requests library is required to browse GitHub files. "
                "Install `textual-universal-directorytree` with the `remote` "
                "extra to install requests."
            ) from e
        url = str(url)
        gitub_prefix = "github://"
        if gitub_prefix in url and "@" not in url:
            _, user_password = url.split("github://")
            org, repo_str = user_password.split(":")
            repo, *args = repo_str.split("/")
        elif gitub_prefix in url and "@" in url:
            return url
        else:
            msg = f"Invalid GitHub URL: {url}"
            raise ValueError(msg)
        token = os.getenv("GITHUB_TOKEN")
        auth = {"auth": ("Bearer", token)} if token is not None else {}
        resp = requests.get(
            f"https://api.github.com/repos/{org}/{repo}",
            headers={"Accept": "application/vnd.github.v3+json"},
            timeout=30,
            **auth,  # type: ignore[arg-type]
        )
        resp.raise_for_status()
        default_branch = resp.json()["default_branch"]
        arg_str = "/".join(args)
        github_uri = f"{gitub_prefix}{org}:{repo}@{default_branch}/{arg_str}"
        return github_uri


class S3TextualPath(S3Path):
    """
    S3TextualPath
    """

    def _is_top_level_bucket(self) -> bool:
        """
        Check if the path is a top level bucket
        """
        return len(self.parts) == 1 and self.parts[0] == "/"

    @property
    def name(self) -> str:
        """
        Override the name for top level repo
        """
        if self._is_top_level_bucket():
            return f"{self._url.scheme}://{self._url.netloc}"
        else:
            return super().name



"""
Helpers for the tests
"""

from __future__ import annotations

import pathlib
from os import environ, getenv

from textual._doc import take_svg_screenshot

from textual_universal_directorytree.app import UniversalDirectoryTreeApp


class Screenshotter:
    """
    App Screenshotter
    """

    def __init__(self, file_path: str | pathlib.Path) -> None:
        """
        Initialize the Screenshotter
        """
        self.app = UniversalDirectoryTreeApp(path=str(file_path))

    def take_screenshot(
        self, press: list[str] | None = None
    ) -> tuple[str, pathlib.Path]:
        """
        Take a Screenshot
        """
        screenshot = take_svg_screenshot(
            app=self.app,
            terminal_size=(120, 35),
            press=press or [],
            title=None,
        )
        screenshot_path = self._get_screenshot_path()
        if getenv("REGENERATE_SCREENSHOTS", "0") != "0":
            screenshot_path.write_text(screenshot)
        return screenshot, screenshot_path

    @classmethod
    def _get_screenshot_path(cls) -> pathlib.Path:
        """
        Get the Screenshot Path

        Screenshots are stored in the docs/screenshots directory
        so they can be included in the documentation
        """
        test_dir = pathlib.Path(__file__).parent
        repo_dir = test_dir.parent
        docs_dir = repo_dir / "docs"
        screenshot_dir = docs_dir / "screenshots"
        current_test = environ["PYTEST_CURRENT_TEST"].split("::")[-1].split(" ")[0]
        screenshot_path = screenshot_dir / f"{current_test}.svg"
        return screenshot_path
